package Inca::Reporter::Performance::Benchmark;

################################################################################

=head1 NAME

Inca::Test::Reporter::Performance::Benchmark - Convenience module for 
managing a single benchmark entry

=head1 SYNOPSIS

  use Inca::Test::Reporter::Performance::Benchmark;
  my $benchmark = new Inca::Test::Reporter::Performance::Benchmark();

=head1 DESCRIPTION

Convenience module for the Inca::Test::Reporter::Performance module.  Manages
a single benchmark entry.

=cut 
################################################################################

use strict;
use warnings;
use Carp;

#=============================================================================#

=head1 CLASS METHODS

=cut
#=============================================================================#

#-----------------------------------------------------------------------------#
# Public methods (documented with pod markers)
#-----------------------------------------------------------------------------#

#-----------------------------------------------------------------------------#

=head2 new

Class constructor which returns a new
Inca::Test::Reporter::Performance::Benchmark object.

=cut
#-----------------------------------------------------------------------------#
sub new {
  my $this = shift;
  my $class = ref($this) || $this;
  my $self = {
    parameters => {},
    statistics => {}
  };
  bless $self, $class;
  return $self;
}

#-----------------------------------------------------------------------------#

=head2 getParameter($name)

=cut
#-----------------------------------------------------------------------------#
sub getParameter {
  my ($self, $name) = @_;
  my $param = $self->{parameters}->{$name};
  return defined($param) ? ($param->{value}, $param->{units}) : undef;
}

#-----------------------------------------------------------------------------#

=head2 getStatistic($name)

=cut
#-----------------------------------------------------------------------------#
sub getStatistic {
  my ($self, $name) = @_;
  my $stat = $self->{statistics}->{$name};
  return defined($stat) ? ($stat->{value}, $stat->{units}) : undef;
}

#-----------------------------------------------------------------------------#

=head2 parameterNames

=cut
#-----------------------------------------------------------------------------#
sub parameterNames {
  my $self = shift;
  return keys %{$self->{parameters}};
}

#-----------------------------------------------------------------------------#

=head2 setParameter($name, $value, $units) 

Sets a parameter that was used to obtain the benchmark.  $name is a unique
identifier for this parameter and $value its value.  The optional $units
describes a scalar $value (e.g., Gb/s, secs, etc.).

=cut
#-----------------------------------------------------------------------------#
sub setParameter {
  my ($self, $name, $value, $units) = @_;
  $self->{parameters}->{$name} = {value => $value, units => $units};
}

#-----------------------------------------------------------------------------#

=head2 setStatistic($name, $value, $units) 

Sets a statistic that was obtained for the benchmark.  $name is a unique
identifier for this parameter and $value its value.  The optional $units
describes a scalar $value (e.g., Gb/s, secs, etc.).

=cut
#-----------------------------------------------------------------------------#
sub setStatistic {
  my ($self, $name, $value, $units) = @_;
  $self->{statistics}->{$name} = {value => $value, units => $units};
}

#-----------------------------------------------------------------------------#

=head2 statisticNames

=cut
#-----------------------------------------------------------------------------#
sub statisticNames {
  my $self = shift;
  return keys %{$self->{statistics}};
}

1;

__END__

=head1 AUTHOR

Shava Smallen <ssmallen@sdsc.edu>

