/* $Id: suite.java,v 1.13 2007/12/12 19:09:08 rbudden Exp $ */

import org.globus.gsi.*;
import org.globus.gsi.*;
import org.globus.gsi.gssapi.*;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl.*;
import org.ietf.jgss.*;
import org.ietf.jgss.GSSCredential.*;
import org.ietf.jgss.GSSException.*;
import org.globus.ftp.*;

import java.lang.*;
import java.util.*;
import java.io.*;

import cogster.common.*;
import cogster.gridftp.*;
import cogster.auth.*;
import cogster.*;

public class suite
{
	private static HashMap conf = new HashMap();
	private static final String FILE = "../config/gridftp.conf";
	/* BUF_SIZE = Mbps * rtt * 1000 / 8 */
	private static final int BUF_SIZE = (int)(1000 * 0.342 * 1000 / 8);

	public static void main(String[] args)
	{
		/* Config File Options */
		final String SRCHOST = "SrcHost";
		final String DSTHOST = "DstHost";
		final String SRCFILE = "SrcFile";
		final String DSTFILE = "DstFile";
		final String GRID_PORT = "GridPort";
		final String PORT_RANGE = "PortRange";

		Cogtime ct = new Cogtime();
		
		System.out.println("----------------------------------");
		System.out.println("---GridFTP Test Suite Beginning---");

		/* Read X509 Cert and create Grid Credentials */
		Auth a = null;
		try
		{
			a = new Auth();
			System.out.println("GridFTP suite executing as user: "+a.getGSS().getName());
		} catch(Exception e) {
			Log("FAILED", "Auth()", e);
		}
		System.out.println("----------------------------------");

		/* Read the config file */
		config();

		/* Set the port range (firewall) */
		String port = conf.get(PORT_RANGE).toString();
		java.lang.System.setProperty("org.globus.tcp.port.range", port);
			
		/* Test connections to GridFTP Servers */
		Gridftp ftp = new Gridftp(a);
		String srch = conf.get(SRCHOST).toString();
		String dsth = conf.get(DSTHOST).toString();
		port = conf.get(GRID_PORT).toString();

		String srcf = conf.get(SRCFILE).toString();
		String dstf = conf.get(DSTFILE).toString();

		try
		{
			/* GSSCredential authentication */
			ftp.connect(srch, Integer.parseInt(port));
			Log("SUCCESS", "GridFTP.authenticate(GSS) on SrcHost: "+srch, null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.authenticate(GSS) on SrcHost: "+srch, e);
			Log("FATAL ERROR", "Connection to GridFTP Server Failed", null);
			System.exit(1);
		}
		System.out.println("----------------------------------");

		/* GridFTP.list test */
		String dir = null;
		try
		{
			Vector v = null;
			v = ftp.list();
			listing(v, "./ on " + srch);
			Log("SUCCESS", "GridFTP.list()", null);

			/* Save a directory if there is one found */
			dir = find_dir(v);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.list()", e);
		}
		System.out.println("----------------------------------");

		/* GridFTP.nlist() */
		try
		{
			if(dir != null)
				ftp.nlist(dir);
			else
				ftp.nlist(null);
			Log("SUCCESS", "GridFTP.nlist(dir)", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.nlist(dir)", e);
		}
		System.out.println("----------------------------------");

		/* GridFTP.mlsd() */
		try
		{
			if(dir != null)
				ftp.mlsd(dir);
			else
				ftp.mlsd(null);
			Log("SUCCESS", "GridFTP.mlsd(dir)", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.mlsd(dir)", e);
		}
		System.out.println("----------------------------------");

		/* test put */
		try
		{
			ct.start();
			ftp.put(srcf, dstf);
			ct.stop();

			ftp.close();
			Log("SUCCESS", "GridFTP.put() - completed in " + ct + " milliseconds", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.put()", e);
		}
		System.out.println("----------------------------------");

		/* Get the file we just transfered */
		try
		{
			ftp.connect(srch, Integer.parseInt(port));

			ct.start();
			ftp.get(dstf, srcf);
			ct.stop();

			ftp.close();
			Log("SUCCESS", "GridFTP.get() - completed in " + ct + " milliseconds", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.get()", e);
		}
		System.out.println("----------------------------------");

/*
** The follow has caused random problems with the transfer hanging in an infinite state.
** Therefore it is initially left disabled. Change the if(false) -> if(true) to test.
** However, initiating via third party extended transfers (as below) did not experience
** the same problems.
*/
if(false)
{
		/* Extended Put File Transfer */
		try
		{
			ftp.connect(srch, Integer.parseInt(port));

			ct.start();
			ftp.setMode(true, true);
			ftp.setbuf(BUF_SIZE);
			ftp.eput(srcf, dstf, 2);
			ct.stop();

			ftp.close();
			Log("SUCCESS", "GridFTP.extendedPut() - completed in " + ct + " milliseconds", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.extendedPut()", e);
		}
		System.out.println("----------------------------------");

		/* Extended Get File Transfer */
		try
		{
			ftp.connect(srch, Integer.parseInt(port));

			ct.start();
			ftp.setMode(true, false);
			ftp.setbuf(BUF_SIZE);
			ftp.eget(dstf, srcf, 2);
			ct.stop();

			ftp.close();
			Log("SUCCESS", "GridFTP.extendedGet() - completed in " + ct + " milliseconds", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.extendedGet()", e);
		}
		System.out.println("----------------------------------");
}

		/* Third party transfer */
		try
		{
			Gridftp ftp2 = new Gridftp(a);
			ftp2.connect(dsth, Integer.parseInt(port));
			ftp.connect(srch, Integer.parseInt(port));

			ct.start();
			ftp.transfer(ftp2, srcf, dstf, 2);

			ct.stop();

			ftp.close();
			ftp2.close();
			Log("SUCCESS", "GridFTP.extendedTransfer() - completed in " + ct + " milliseconds", null);
		} catch(Exception e) {
			Log("FAILED", "GridFTP.extendedTransfer()", e);
		}
		System.out.println("----------------------------------");

		Log("COMPLETED", "GridFTP Test Suite", null);
		System.out.println("----------------------------------");
	}

	/* Read the GridFTP config files and prep for test suite */
	public static void config()
	{
		try
		{
			File f = new File(FILE);
			BufferedReader fin = new BufferedReader(new FileReader(f));
			String line;
			String[] field;

			/*
			** Parse the config file and ignore empty
			** lines and lines beginning with #
			*/
			while((line = fin.readLine()) != null)
			{
				if(line.compareTo("") != 0 && !line.startsWith("#"))
				{
					field = line.split(" ");
					conf.put(field[0], field[1]);
				}
			}
		} catch(Exception e) {
			System.out.println("ERROR!! Problems reading gridftp config file: " + e);
			System.exit(1);
		}
	}

	/* Print the contents of a directory listing (FileInfo) vector */
	public static void listing(Vector v, String dir)
	{
		FileInfo f;

		System.out.println("\t\tDirectory Listing for '"+dir+"':");
		for(int i = 0; i < v.size(); i++)
		{
			f = (FileInfo)(v.get(i));
			System.out.println("\t\t" + f.getName());
		}
		System.out.println("\n");
	}

	/* Find the first directory in a FileInfo Vector */
	public static String find_dir(Vector v)
	{
		FileInfo f;
		String dir = null;

		for(int i = 0; i < v.size(); i++)
		{
			f = (FileInfo)(v.get(i));

			/* Make sure we don't get . or .. as directories */
			if(f.isDirectory() && f.getName() != "." && f.getName() != "..")
			{
				dir = f.getName();
				break;
			}
		}

		return dir;
	}

	public static void Log(String action, String msg, Exception err)
	{
		System.out.println("---" + action + " " + msg);
		if(err != null)
			System.err.println(err);
	}
}


