/* $Id: suite.java,v 1.3 2007/12/12 19:09:48 rbudden Exp $ */

import org.globus.gsi.*;
import org.globus.gsi.*;
import org.globus.gsi.gssapi.*;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl.*;
import org.ietf.jgss.*;
import org.ietf.jgss.GSSCredential.*;
import org.ietf.jgss.GSSException.*;
import org.globus.ftp.*;
import org.globus.io.urlcopy.*;
import org.globus.util.*;

import java.lang.*;
import java.util.*;
import java.io.*;

import cogster.common.*;
import cogster.gridftp.*;
import cogster.auth.*;
import cogster.*;

public class suite
{
	private static HashMap conf = new HashMap();
	private static final String CONF = "../config/urlcopy.conf";

	public static void main(String[] args)
	{
		/* Config File Options */
		final String SRC_FILE = "SrcFile";
		final String DST_FILE = "DstFile";
		final String SRC_HOST = "SrcHost";
		final String DST_HOST = "DstHost";
		final String PORT_RANGE = "PortRange";

		GlobusURL srctu = null;
		GlobusURL dsttu = null;
		GlobusURL srcu = null;
		GlobusURL dstu = null;

		Cogtime ct = new Cogtime();
		UrlCopy uc = new UrlCopy();
		
		System.out.println("----------------------------------");
		System.out.println("---UrlCopy Test Suite Beginning---");

		/* Read X509 Cert and create Grid Credentials */
		Auth a = null;
		try
		{
			a = new Auth();
			System.out.println("Suite executing as user: "+a.getGSS().getName());
		} catch(Exception e) {
			Log("FAILED", "Auth()", e);
		}
		System.out.println("----------------------------------");

		/* Read the config file */
		config();

		/* Set the port range (firewall) for PSC */
		String port = conf.get(PORT_RANGE).toString();
		java.lang.System.setProperty("org.globus.tcp.port.range", port);
			
		uc.setCredentials(a.getGSS());

		/* build the globus urls */
		String srcf = conf.get(SRC_FILE).toString();
		String dstf = conf.get(DST_FILE).toString();
		String srch = conf.get(SRC_HOST).toString();
		String dsth = conf.get(DST_HOST).toString();

		try
		{
			/* XXX - urls need extra / before path for some reason */

			/* third party type urls */
			srctu = new GlobusURL("gsiftp://" + srch + "/" + srcf);
			dsttu = new GlobusURL("gsiftp://" + dsth + "/" + dstf);

			/* non third party type (does this get done automatically?) */
			srcu = new GlobusURL("file:///" + srcf);
			dstu = new GlobusURL("gsiftp://" + dsth + "/" + dstf);

			System.out.println("Transfer:");
			System.out.println(srcu.getURL() + " -> " + dstu.getURL());
			System.out.println(srctu.getURL() + " -> " + dsttu.getURL());

			/* non third party first */
			uc.setSourceUrl(srcu);
			uc.setDestinationUrl(dstu);
		} catch(Exception e) {
			System.out.println("Error: " + e.toString());
		}

		/* try different variations of urlcopies */
		try
		{
			uc.setUseThirdPartyCopy(false);
			uc.setDCAU(false);
			uc.copy();
			Log("SUCCESS", "UrlCopy: ThirdParth=false DCAU=false", null);
		} catch(Exception e) {
			Log("FAILED", "UrlCopy: ThirdParth=false DCAU=false", e);
		}
		System.out.println("----------------------------------");

		try
		{
			uc.setUseThirdPartyCopy(false);
			uc.setDCAU(true);
			uc.copy();
			Log("SUCCESS", "UrlCopy: ThirdParth=false DCAU=true", null);
		} catch(Exception e) {
			Log("FAILED", "UrlCopy: ThirdParth=false DCAU=true", e);
		}
		System.out.println("----------------------------------");

		/* set for third party urls */
		try
		{
			uc.setSourceUrl(srctu);
			uc.setDestinationUrl(dsttu);
		} catch(Exception e) {
			System.out.println("Error: " + e.toString());
		}
	
		try
		{
			uc.setUseThirdPartyCopy(true);
			uc.setDCAU(false);
			uc.copy();
			Log("SUCCESS", "UrlCopy: ThirdParth=true DCAU=false", null);
		} catch (Exception e) {
			Log("FAILED", "UrlCopy: ThirdParth=true DCAU=false", e);
		}
		System.out.println("----------------------------------");

		try
		{	
			uc.setUseThirdPartyCopy(true);
			uc.setDCAU(true);
			uc.copy();
			Log("SUCCESS", "UrlCopy: ThirdParth=true DCAU=true", null);
		} catch (Exception e) {
			Log("FAILED", "UrlCopy: ThirdParth=true DCAU=true", e);
		}
		System.out.println("----------------------------------");
	}

	/* Read the config files and prep for test suite */
	public static void config()
	{
		try
		{
			File f = new File(CONF);
			BufferedReader fin = new BufferedReader(new FileReader(f));
			String line;
			String[] field;

			/*
			** Parse the config file and ignore empty
			** lines and lines beginning with #
			*/
			while((line = fin.readLine()) != null)
			{
				if(line.compareTo("") != 0 && !line.startsWith("#"))
				{
					field = line.split(" ");
					conf.put(field[0], field[1]);
				}
			}
		} catch(Exception e) {
			System.out.println("ERROR!! Problems reading config file: " + e);
			System.exit(1);
		}
	}

	public static void Log(String action, String msg, Exception e)
	{
		System.out.println("---" + action + " " + msg);
		if(e != null)
			System.err.println(e);
	}
}


